@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2022 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -19,6 +19,7 @@
  **/
 
 #include "common/version.h"
+#include "common/runtime.h"
 #include "modules/love/love.h"
 #include <SDL.h>
 
@@ -44,17 +45,10 @@ extern "C" {
 #include "common/ios.h"
 #endif
 
-#ifdef LOVE_ANDROID
-#include "common/android.h"
-extern "C" 
-{
-#include "luajit.h"
-}
-#endif
-
 #ifdef LOVE_WINDOWS
 extern "C"
 {
+
 // Prefer the higher performance GPU on Windows systems that use nvidia Optimus.
 // http://developer.download.nvidia.com/devzone/devcenter/gamegraphics/files/OptimusRenderingPolicies.pdf
 // TODO: Re-evaluate in the future when the average integrated GPU in Optimus
@@ -81,38 +75,39 @@ static void get_app_arguments(int argc, char **argv, int &new_argc, char **&new_
 			temp_argv.push_back(std::string(argv[i]));
 	}
 
-#ifdef LOVE_MACOSX
-	// Check for a drop file string, if the app wasn't launched in a terminal.
-	// Checking for the terminal is a pretty big hack, but works around an issue
-	// where OS X will switch Spaces if the terminal launching love is in its
-	// own full-screen Space.
-	std::string dropfilestr;
-	if (!isatty(STDIN_FILENO))
-		dropfilestr = love::macosx::checkDropEvents();
-
-	if (!dropfilestr.empty())
-		temp_argv.insert(temp_argv.begin() + 1, dropfilestr);
-	else
-#endif
-	{
-		// If it exists, add the love file in love.app/Contents/Resources/ to argv.
-		std::string loveResourcesPath;
-		bool fused = true;
+	// If it exists, add the love file in love.app/Contents/Resources/ to argv.
+	std::string loveResourcesPath;
+	bool fused = true;
 #if defined(LOVE_MACOSX)
-		loveResourcesPath = love::macosx::getLoveInResources();
+	loveResourcesPath = love::macosx::getLoveInResources();
 #elif defined(LOVE_IOS)
-		loveResourcesPath = love::ios::getLoveInResources(fused);
+	loveResourcesPath = love::ios::getLoveInResources(fused);
 #endif
-		if (!loveResourcesPath.empty())
-		{
-			std::vector<std::string>::iterator it = temp_argv.begin();
-			it = temp_argv.insert(it + 1, loveResourcesPath);
+	if (!loveResourcesPath.empty())
+	{
+		std::vector<std::string>::iterator it = temp_argv.begin();
+		it = temp_argv.insert(it + 1, loveResourcesPath);
 
-			// Run in pseudo-fused mode.
-			if (fused)
-				temp_argv.insert(it + 1, std::string("--fused"));
+		// Run in pseudo-fused mode.
+		if (fused)
+			temp_argv.insert(it + 1, std::string("--fused"));
+	}
+#ifdef LOVE_MACOSX
+	else
+	{
+		// Check for a drop file string, if the app wasn't launched in a
+		// terminal. Checking for the terminal is a pretty big hack, but works
+		// around an issue where OS X will switch Spaces if the terminal
+		// launching love is in its own full-screen Space.
+		if (!isatty(STDIN_FILENO))
+		{
+			// Static to keep the same value after love.event.equit("restart").
+			static std::string dropfilestr = love::macosx::checkDropEvents();
+			if (!dropfilestr.empty())
+				temp_argv.insert(temp_argv.begin() + 1, dropfilestr);
 		}
 	}
+#endif
 
 	// Copy temp argv vector to new argv array.
 	new_argc = (int) temp_argv.size();
@@ -139,39 +134,6 @@ static int love_preload(lua_State *L, lua_CFunction f, const char *name)
 	return 0;
 }
 
-#ifdef LOVE_ANDROID
-static int l_print_sdl_log(lua_State *L)
-{
-	int nargs = lua_gettop(L);
-
-	lua_getglobal(L, "tostring");
-
-	std::string outstring;
-
-	for (int i = 1; i <= nargs; i++)
-	{
-		// Call tostring(arg) and leave the result on the top of the stack.
-		lua_pushvalue(L, -1);
-		lua_pushvalue(L, i);
-		lua_call(L, 1, 1);
-
-		const char *s = lua_tostring(L, -1);
-		if (s == nullptr)
-			return luaL_error(L, "'tostring' must return a string to 'print'");
-
-		if (i > 1)
-			outstring += "\t";
-
-		outstring += s;
-
-		lua_pop(L, 1); // Pop the result of tostring(arg).
-	}
-
-	SDL_Log("[LOVE] %s", outstring.c_str());
-	return 0;
-}
-#endif
-
 enum DoneAction
 {
 	DONE_QUIT,
@@ -180,17 +142,13 @@ enum DoneAction
 
 static DoneAction runlove(int argc, char **argv, int &retval)
 {
-#ifdef LOVE_LEGENDARY_APP_ARGV_HACK
-	int hack_argc = 0;
-	char **hack_argv = 0;
-	get_app_arguments(argc, argv, hack_argc, hack_argv);
-	argc = hack_argc;
-	argv = hack_argv;
-#endif // LOVE_LEGENDARY_APP_ARGV_HACK
-
 	// Oh, you just want the version? Okay!
 	if (argc > 1 && strcmp(argv[1], "--version") == 0)
 	{
+#ifdef LOVE_LEGENDARY_CONSOLE_IO_HACK
+		const char *err = nullptr;
+		love_openConsole(err);
+#endif
 		printf("LOVE %s (%s)\n", love_version(), love_codename());
 		retval = 0;
 		return DONE_QUIT;
@@ -200,10 +158,21 @@ static DoneAction runlove(int argc, char **argv, int &retval)
 	lua_State *L = luaL_newstate();
 	luaL_openlibs(L);
 
-#ifdef LOVE_ANDROID
-	luaJIT_setmode(L, 0, LUAJIT_MODE_ENGINE| LUAJIT_MODE_OFF);
-	lua_register(L, "print", l_print_sdl_log);
-#endif
+	// LuaJIT-specific setup needs to be done as early as possible - before
+	// get_app_arguments because that loads external library code. This is also
+	// loaded inside require("love"). Note that it doesn't use the love table.
+	love_preload(L, luaopen_love_jitsetup, "love.jitsetup");
+	lua_getglobal(L, "require");
+	lua_pushstring(L, "love.jitsetup");
+	lua_call(L, 1, 0);
+
+#ifdef LOVE_LEGENDARY_APP_ARGV_HACK
+	int hack_argc = 0;
+	char **hack_argv = nullptr;
+	get_app_arguments(argc, argv, hack_argc, hack_argv);
+	argc = hack_argc;
+	argv = hack_argv;
+#endif // LOVE_LEGENDARY_APP_ARGV_HACK
 
 	// Add love to package.preload for easy requiring.
 	love_preload(L, luaopen_love, "love");
@@ -251,8 +220,17 @@ static DoneAction runlove(int argc, char **argv, int &retval)
 	lua_pushstring(L, "love.boot");
 	lua_call(L, 1, 1);
 
-	// Call the returned boot function.
-	lua_call(L, 0, 1);
+	// Turn the returned boot function into a coroutine and call it until done.
+	lua_newthread(L);
+	lua_pushvalue(L, -2);
+	int stackpos = lua_gettop(L);
+	int nres;
+	while (love::luax_resume(L, 0, &nres) == LUA_YIELD)
+#if LUA_VERSION_NUM >= 504
+		lua_pop(L, nres);
+#else
+		lua_pop(L, lua_gettop(L) - stackpos);
+#endif
 
 	retval = 0;
 	DoneAction done = DONE_QUIT;
