@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2022 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -18,11 +18,10 @@
  * 3. This notice may not be removed or altered from any source distribution.
  **/
 
-#ifndef LOVE_IMAGE_MAGPIE_KTX_HANDLER_H
-#define LOVE_IMAGE_MAGPIE_KTX_HANDLER_H
+#pragma once
 
 #include "common/config.h"
-#include "CompressedFormatHandler.h"
+#include "image/FormatHandler.h"
 
 namespace love
 {
@@ -34,20 +33,21 @@ namespace magpie
 /**
  * Handles KTX files with compressed image data inside.
  **/
-class KTXHandler : public CompressedFormatHandler
+class KTXHandler : public FormatHandler
 {
 public:
 
 	virtual ~KTXHandler() {}
 
-	// Implements CompressedFormatHandler.
-	virtual bool canParse(const filesystem::FileData *data);
-	virtual uint8 *parse(filesystem::FileData *filedata, std::vector<CompressedImageData::SubImage> &images, size_t &dataSize, CompressedImageData::Format &format, bool &sRGB);
+	// Implements FormatHandler.
+	bool canParseCompressed(Data *data) override;
+
+	StrongRef<CompressedMemory> parseCompressed(Data *filedata,
+	        std::vector<StrongRef<CompressedSlice>> &images,
+	        PixelFormat &format, bool &sRGB) override;
 
 }; // KTXHandler
 
 } // magpie
 } // image
 } // love
-
-#endif // LOVE_IMAGE_MAGPIE_KTX_HANDLER_H
