@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2022 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -21,8 +21,9 @@
 // LOVE
 #include "common/config.h"
 #include "common/version.h"
+#include "common/deprecation.h"
 #include "common/runtime.h"
-#include "common/wrap_Data.h"
+#include "modules/window/Window.h"
 
 #include "love.h"
 
@@ -32,8 +33,22 @@
 
 #ifdef LOVE_WINDOWS
 #include <windows.h>
+
+#if defined(_MSC_VER) && (_MSC_VER < 1900)
+// VS 2013 and earlier doesn't have snprintf
+#define snprintf sprintf_s
+#endif // defined(_MSC_VER) && (_MSC_VER < 1900)
+
 #endif // LOVE_WINDOWS
 
+#ifdef LOVE_ANDROID
+#include <SDL.h>
+extern "C"
+{
+#include "luajit.h"
+}
+#endif // LOVE_ANDROID
+
 #ifdef LOVE_LEGENDARY_CONSOLE_IO_HACK
 #include <fcntl.h>
 #include <io.h>
@@ -52,8 +67,8 @@
 #ifdef LOVE_ENABLE_ENET
 #	include "libraries/enet/lua-enet.h"
 #endif
-#ifdef LOVE_ENABLE_LUAUTF8
-#	include "libraries/luautf8/lutf8lib.h"
+#ifdef LOVE_ENABLE_LUA53
+#	include "libraries/lua53/lutf8lib.h"
 #endif
 
 // For love::graphics::setGammaCorrect.
@@ -61,9 +76,30 @@
 #	include "graphics/Graphics.h"
 #endif
 
-// Scripts
+// For love::audio::Audio::setMixWithSystem.
+#ifdef LOVE_ENABLE_AUDIO
+#	include "audio/Audio.h"
+#endif
+
+// Scripts.
 #include "scripts/nogame.lua.h"
-#include "scripts/boot.lua.h"
+
+// Put the Lua code directly into a raw string literal.
+static const char arg_lua[] =
+#include "arg.lua"
+;
+
+static const char callbacks_lua[] =
+#include "callbacks.lua"
+;
+
+static const char boot_lua[] =
+#include "boot.lua"
+;
+
+static const char jit_setup_lua[] =
+#include "jitsetup.lua"
+;
 
 // All modules define a c-accessible luaopen
 // so let's make use of those, instead
@@ -73,6 +109,9 @@ extern "C"
 #if defined(LOVE_ENABLE_AUDIO)
 	extern int luaopen_love_audio(lua_State*);
 #endif
+#if defined(LOVE_ENABLE_DATA)
+	extern int luaopen_love_data(lua_State*);
+#endif
 #if defined(LOVE_ENABLE_EVENT)
 	extern int luaopen_love_event(lua_State*);
 #endif
@@ -125,6 +164,9 @@ extern "C"
 	extern int luaopen_love_window(lua_State*);
 #endif
 	extern int luaopen_love_nogame(lua_State*);
+	extern int luaopen_love_jitsetup(lua_State*);
+	extern int luaopen_love_arg(lua_State*);
+	extern int luaopen_love_callbacks(lua_State*);
 	extern int luaopen_love_boot(lua_State*);
 }
 
@@ -132,6 +174,9 @@ static const luaL_Reg modules[] = {
 #if defined(LOVE_ENABLE_AUDIO)
 	{ "love.audio", luaopen_love_audio },
 #endif
+#if defined(LOVE_ENABLE_DATA)
+	{ "love.data", luaopen_love_data },
+#endif
 #if defined(LOVE_ENABLE_EVENT)
 	{ "love.event", luaopen_love_event },
 #endif
@@ -168,12 +213,12 @@ static const luaL_Reg modules[] = {
 #if defined(LOVE_ENABLE_SYSTEM)
 	{ "love.system", luaopen_love_system },
 #endif
-#if defined(LOVE_ENABLE_TIMER)
-	{ "love.timer", luaopen_love_timer },
-#endif
 #if defined(LOVE_ENABLE_THREAD)
 	{ "love.thread", luaopen_love_thread },
 #endif
+#if defined(LOVE_ENABLE_TIMER)
+	{ "love.timer", luaopen_love_timer },
+#endif
 #if defined(LOVE_ENABLE_TOUCH)
 	{ "love.touch", luaopen_love_touch },
 #endif
@@ -184,6 +229,9 @@ static const luaL_Reg modules[] = {
 	{ "love.window", luaopen_love_window },
 #endif
 	{ "love.nogame", luaopen_love_nogame },
+	{ "love.jitsetup", luaopen_love_jitsetup },
+	{ "love.arg", luaopen_love_arg },
+	{ "love.callbacks", luaopen_love_callbacks },
 	{ "love.boot", luaopen_love_boot },
 	{ 0, 0 }
 };
@@ -196,6 +244,39 @@ int w__openConsole(lua_State *L);
 int w__setAccelerometerAsJoystick(lua_State *L);
 #endif
 
+#ifdef LOVE_ANDROID
+static int w_print_sdl_log(lua_State *L)
+{
+	int nargs = lua_gettop(L);
+
+	lua_getglobal(L, "tostring");
+
+	std::string outstring;
+
+	for (int i = 1; i <= nargs; i++)
+	{
+		// Call tostring(arg) and leave the result on the top of the stack.
+		lua_pushvalue(L, -1);
+		lua_pushvalue(L, i);
+		lua_call(L, 1, 1);
+
+		const char *s = lua_tostring(L, -1);
+		if (s == nullptr)
+			return luaL_error(L, "'tostring' must return a string to 'print'");
+
+		if (i > 1)
+			outstring += "\t";
+
+		outstring += s;
+
+		lua_pop(L, 1); // Pop the result of tostring(arg).
+	}
+
+	SDL_Log("[LOVE] %s", outstring.c_str());
+	return 0;
+}
+#endif
+
 const char *love_version()
 {
 	// Do not refer to love::VERSION here, the linker
@@ -222,12 +303,18 @@ static int w_love_isVersionCompatible(lua_State *L)
 	std::string version;
 
 	if (lua_type(L, 1) == LUA_TSTRING)
+	{
 		version = luaL_checkstring(L, 1);
+
+		// Convert major.minor to major.minor.revision.
+		if (std::count(version.begin(), version.end(), '.') < 2)
+			version.append(".0");
+	}
 	else
 	{
-		int major = (int) luaL_checknumber(L, 1);
-		int minor = (int) luaL_checknumber(L, 2);
-		int rev   = (int) luaL_checknumber(L, 3);
+		int major = (int) luaL_checkinteger(L, 1);
+		int minor = (int) luaL_checkinteger(L, 2);
+		int rev   = (int) luaL_optinteger(L, 3, 0);
 
 		// Convert the numbers to a string, since VERSION_COMPATIBILITY is an
 		// array of version strings.
@@ -239,7 +326,13 @@ static int w_love_isVersionCompatible(lua_State *L)
 
 	for (int i = 0; love::VERSION_COMPATIBILITY[i] != nullptr; i++)
 	{
-		if (version.compare(love::VERSION_COMPATIBILITY[i]) != 0)
+		std::string v(love::VERSION_COMPATIBILITY[i]);
+
+		// Convert major.minor to major.minor.revision.
+		if (std::count(v.begin(), v.end(), '.') < 2)
+			v.append(".0");
+
+		if (version.compare(v) != 0)
 			continue;
 
 		lua_pushboolean(L, true);
@@ -258,8 +351,74 @@ static int w__setGammaCorrect(lua_State *L)
 	return 0;
 }
 
+static int w__setAudioMixWithSystem(lua_State *L)
+{
+	bool success = false;
+
+#ifdef LOVE_ENABLE_AUDIO
+	bool mix = love::luax_checkboolean(L, 1);
+	success = love::audio::Audio::setMixWithSystem(mix);
+#endif
+
+	love::luax_pushboolean(L, success);
+	return 1;
+}
+
+static int w__requestRecordingPermission(lua_State *L)
+{
+#ifdef LOVE_ENABLE_AUDIO
+	love::audio::setRequestRecordingPermission((bool) lua_toboolean(L, 1));
+#endif
+	return 0;
+}
+
+static int w_love_setDeprecationOutput(lua_State *L)
+{
+	bool enable = love::luax_checkboolean(L, 1);
+	love::setDeprecationOutputEnabled(enable);
+	return 0;
+}
+
+static int w_love_hasDeprecationOutput(lua_State *L)
+{
+	love::luax_pushboolean(L, love::isDeprecationOutputEnabled());
+	return 1;
+}
+
+static int w_deprecation__gc(lua_State *)
+{
+	love::deinitDeprecation();
+	return 0;
+}
+
+static void luax_addcompatibilityalias(lua_State *L, const char *module, const char *name, const char *alias)
+{
+	lua_getglobal(L, module);
+	if (lua_istable(L, -1))
+	{
+		lua_getfield(L, -1, alias);
+		bool hasalias = !lua_isnoneornil(L, -1);
+		lua_pop(L, 1);
+		if (!hasalias)
+		{
+			lua_getfield(L, -1, name);
+			lua_setfield(L, -2, alias);
+		}
+	}
+	lua_pop(L, 1);
+}
+
 int luaopen_love(lua_State *L)
 {
+	// Preload module loaders.
+	for (int i = 0; modules[i].name != nullptr; i++)
+		love::luax_preload(L, modules[i].func, modules[i].name);
+
+	// jitsetup is also loaded in the love executable runlove function. It's
+	// needed here too for threads. Note that it doesn't use the love table.
+	love::luax_require(L, "love.jitsetup");
+	lua_pop(L, 1);
+
 	love::luax_insistpinnedthread(L);
 
 	love::luax_insistglobal(L, "love");
@@ -278,6 +437,11 @@ int luaopen_love(lua_State *L)
 	lua_pushstring(L, love::VERSION_CODENAME);
 	lua_setfield(L, -2, "_version_codename");
 
+#ifdef LOVE_ANDROID
+	luaJIT_setmode(L, 0, LUAJIT_MODE_ENGINE | LUAJIT_MODE_OFF);
+	lua_register(L, "print", w_print_sdl_log);
+#endif
+
 #ifdef LOVE_LEGENDARY_CONSOLE_IO_HACK
 	lua_pushcfunction(L, w__openConsole);
 	lua_setfield(L, -2, "_openConsole");
@@ -291,6 +455,13 @@ int luaopen_love(lua_State *L)
 	lua_pushcfunction(L, w__setGammaCorrect);
 	lua_setfield(L, -2, "_setGammaCorrect");
 
+	// Exposed here because we need to be able to call it before the audio
+	// module is initialized.
+	lua_pushcfunction(L, w__setAudioMixWithSystem);
+	lua_setfield(L, -2, "_setAudioMixWithSystem");
+	lua_pushcfunction(L, w__requestRecordingPermission);
+	lua_setfield(L, -2, "_requestRecordingPermission");
+
 	lua_newtable(L);
 
 	for (int i = 0; love::VERSION_COMPATIBILITY[i] != nullptr; i++)
@@ -324,12 +495,37 @@ int luaopen_love(lua_State *L)
 #endif
 	lua_setfield(L, -2, "_os");
 
-	// Preload module loaders.
-	for (int i = 0; modules[i].name != nullptr; i++)
-		love::luax_preload(L, modules[i].func, modules[i].name);
+	{
+		love::initDeprecation();
 
-	// Load "common" types.
-	love::w_Data_open(L);
+		// Any old data that we can attach a metatable to, for __gc. We want to
+		// call deinitDeprecation when love is garbage collected.
+		lua_newuserdata(L, sizeof(int));
+
+		luaL_newmetatable(L, "love_deprecation");
+		lua_pushcfunction(L, w_deprecation__gc);
+		lua_setfield(L, -2, "__gc");
+		lua_setmetatable(L, -2);
+
+		lua_setfield(L, -2, "_deprecation");
+
+		lua_pushcfunction(L, w_love_setDeprecationOutput);
+		lua_setfield(L, -2, "setDeprecationOutput");
+
+		lua_pushcfunction(L, w_love_hasDeprecationOutput);
+		lua_setfield(L, -2, "hasDeprecationOutput");
+	}
+
+	// Necessary for Data-creating methods to work properly in Data subclasses.
+	love::luax_require(L, "love.data");
+	lua_pop(L, 1);
+
+#if LUA_VERSION_NUM <= 501
+	// These are deprecated in Lua 5.1. LuaJIT 2.1 removes them, but code
+	// written assuming LuaJIT 2.0 or Lua 5.1 is used might still rely on them.
+	luax_addcompatibilityalias(L, "math", "fmod", "mod");
+	luax_addcompatibilityalias(L, "string", "gmatch", "gfind");
+#endif
 
 #ifdef LOVE_ENABLE_LUASOCKET
 	love::luasocket::__open(L);
@@ -337,54 +533,70 @@ int luaopen_love(lua_State *L)
 #ifdef LOVE_ENABLE_ENET
 	love::luax_preload(L, luaopen_enet, "enet");
 #endif
-#ifdef LOVE_ENABLE_LUAUTF8
+#ifdef LOVE_ENABLE_LUA53
 	love::luax_preload(L, luaopen_luautf8, "utf8");
 #endif
 
+#ifdef LOVE_ENABLE_WINDOW
+	// In some environments, LuaJIT is limited to 2GB and LuaJIT sometimes panic when it
+	// reaches OOM and closes the whole program, leaving the user confused about what's
+	// going on.
+	// We can't recover the state at this point, but it's better to inform user that
+	// something very bad happening instead of silently exiting.
+	// Note that this is not foolproof. In some cases, the whole process crashes by
+	// uncaught exception that LuaJIT throws or simply exit as if calling
+	// love.event.quit("not enough memory")
+	lua_atpanic(L, [](lua_State *L)
+	{
+		using namespace love;
+		using namespace love::window;
+
+		char message[128];
+		Window* windowModule = Module::getInstance<Window>(Module::M_WINDOW);
+
+		snprintf(message, sizeof(message), "PANIC: unprotected error in call to Lua API (%s)", lua_tostring(L, -1));
+
+		if (windowModule)
+			windowModule->showMessageBox("Lua Fatal Error", message, Window::MESSAGEBOX_ERROR, windowModule->isOpen());
+
+		fprintf(stderr, "%s\n", message);
+		return 0;
+	});
+#endif
+
 	return 1;
 }
 
 #ifdef LOVE_LEGENDARY_CONSOLE_IO_HACK
 
-// Mostly taken from the Windows 8.1 SDK's VersionHelpers.h.
-static bool IsWindowsVistaOrGreater()
-{
-	OSVERSIONINFOEXW osvi = {sizeof(osvi), 0, 0, 0, 0, {0}, 0, 0};
-
-	osvi.dwMajorVersion = HIBYTE(_WIN32_WINNT_VISTA);
-	osvi.dwMinorVersion = LOBYTE(_WIN32_WINNT_VISTA);
-	osvi.wServicePackMajor = 0;
-
-	DWORDLONG majorversionmask = VerSetConditionMask(0, VER_MAJORVERSION, VER_GREATER_EQUAL);
-	DWORDLONG versionmask = VerSetConditionMask(majorversionmask, VER_MINORVERSION, VER_GREATER_EQUAL);
-	DWORDLONG mask = VerSetConditionMask(versionmask, VER_SERVICEPACKMAJOR, VER_GREATER_EQUAL);
-
-	return VerifyVersionInfoW(&osvi, VER_MAJORVERSION | VER_MINORVERSION | VER_SERVICEPACKMAJOR, mask) != FALSE;
-}
-
-int w__openConsole(lua_State *L)
+bool love_openConsole(const char *&err)
 {
 	static bool is_open = false;
+	err = nullptr;
 
 	if (is_open)
-	{
-		love::luax_pushboolean(L, is_open);
-		return 1;
-	}
+		return true;
 
 	is_open = true;
 
-	// FIXME: we don't call AttachConsole in Windows XP because it seems to
-	// break later AllocConsole calls if it fails. A better workaround might be
-	// possible, but it's hard to find a WinXP system to test on these days...
-	if (!IsWindowsVistaOrGreater() || !AttachConsole(ATTACH_PARENT_PROCESS))
+	if (!AttachConsole(ATTACH_PARENT_PROCESS))
 	{
+		DWORD winerr = GetLastError();
+
+		if (winerr == ERROR_ACCESS_DENIED)
+		{
+			// The process is already attached to a console. We'll assume stdout
+			// and friends are already being directed there.
+			is_open = true;
+			return is_open;
+		}
+
 		// Create our own console if we can't attach to an existing one.
 		if (!AllocConsole())
 		{
 			is_open = false;
-			love::luax_pushboolean(L, is_open);
-			return 1;
+			err = "Could not create console.";
+			return is_open;
 		}
 
 		SetConsoleTitle(TEXT("LOVE Console"));
@@ -402,20 +614,38 @@ int w__openConsole(lua_State *L)
 
 	// Redirect stdout.
 	fp = freopen("CONOUT$", "w", stdout);
-	if (L && fp == NULL)
-		return luaL_error(L, "Console redirection of stdout failed.");
+	if (fp == NULL)
+	{
+		err = "Console redirection of stdout failed.";
+		return is_open;
+	}
 
 	// Redirect stdin.
 	fp = freopen("CONIN$", "r", stdin);
-	if (L && fp == NULL)
-		return luaL_error(L, "Console redirection of stdin failed.");
+	if (fp == NULL)
+	{
+		err = "Console redirection of stdin failed.";
+		return is_open;
+	}
 
 	// Redirect stderr.
 	fp = freopen("CONOUT$", "w", stderr);
-	if (L && fp == NULL)
-		return luaL_error(L, "Console redirection of stderr failed.");
+	if (fp == NULL)
+	{
+		err = "Console redirection of stderr failed.";
+		return is_open;
+	}
 
-	love::luax_pushboolean(L, is_open);
+	return is_open;
+}
+
+int w__openConsole(lua_State *L)
+{
+	const char *err = nullptr;
+	bool isopen = love_openConsole(err);
+	if (err != nullptr)
+		return luaL_error(L, err);
+	love::luax_pushboolean(L, isopen);
 	return 1;
 }
 
@@ -432,7 +662,31 @@ int w__setAccelerometerAsJoystick(lua_State *L)
 
 int luaopen_love_nogame(lua_State *L)
 {
-	if (luaL_loadbuffer(L, (const char *)love::nogame_lua, sizeof(love::nogame_lua), "nogame.lua") == 0)
+	if (luaL_loadbuffer(L, (const char *)love::nogame_lua, sizeof(love::nogame_lua), "=[love \"nogame.lua\"]") == 0)
+		lua_call(L, 0, 1);
+
+	return 1;
+}
+
+int luaopen_love_jitsetup(lua_State *L)
+{
+	if (luaL_loadbuffer(L, jit_setup_lua, sizeof(jit_setup_lua), "=[love \"jitsetup.lua\"]") == 0)
+		lua_call(L, 0, 1);
+
+	return 1;
+}
+
+int luaopen_love_arg(lua_State *L)
+{
+	if (luaL_loadbuffer(L, arg_lua, sizeof(arg_lua), "=[love \"arg.lua\"]") == 0)
+		lua_call(L, 0, 1);
+
+	return 1;
+}
+
+int luaopen_love_callbacks(lua_State *L)
+{
+	if (luaL_loadbuffer(L, callbacks_lua, sizeof(callbacks_lua), "=[love \"callbacks.lua\"]") == 0)
 		lua_call(L, 0, 1);
 
 	return 1;
@@ -440,11 +694,8 @@ int luaopen_love_nogame(lua_State *L)
 
 int luaopen_love_boot(lua_State *L)
 {
-	if (luaL_loadbuffer(L, (const char *)love::boot_lua, sizeof(love::boot_lua), "boot.lua") == 0)
+	if (luaL_loadbuffer(L, boot_lua, sizeof(boot_lua), "=[love \"boot.lua\"]") == 0)
 		lua_call(L, 0, 1);
 
 	return 1;
 }
-
-
-
