@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2022 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -23,7 +23,6 @@
 // Module
 #include "Body.h"
 #include "World.h"
-#include "common/Memoizer.h"
 
 namespace love
 {
@@ -32,6 +31,8 @@ namespace physics
 namespace box2d
 {
 
+love::Type GearJoint::type("GearJoint", &Joint::type);
+
 GearJoint::GearJoint(Joint *joint1, Joint *joint2, float ratio, bool collideConnected)
 	: Joint(joint1->body2, joint2->body2)
 	, joint(NULL)
@@ -67,7 +68,7 @@ Joint *GearJoint::getJointA() const
 	if (b2joint == nullptr)
 		return nullptr;
 
-	Joint *j = (Joint *) Memoizer::find(b2joint);
+	Joint *j = (Joint *) world->findObject(b2joint);
 	if (j == nullptr)
 		throw love::Exception("A joint has escaped Memoizer!");
 
@@ -80,7 +81,7 @@ Joint *GearJoint::getJointB() const
 	if (b2joint == nullptr)
 		return nullptr;
 
-	Joint *j = (Joint *) Memoizer::find(b2joint);
+	Joint *j = (Joint *) world->findObject(b2joint);
 	if (j == nullptr)
 		throw love::Exception("A joint has escaped Memoizer!");
 
